<?php

namespace InSegment\WorkersFramework\Console;

use Shavshukov\EventAppBase\Application as BaseApplication;
use Shavshukov\EventAppBase\Interfaces\ModelInterface;

use InSegment\WorkersFramework\Models\Config;

class Application extends BaseApplication
{
    /**
     * Filename of config file
     *
     * @var string
     */
    protected $configFileName;
    
    /**
     * Constructor
     * 
     * @param string $appPath
     * @param string $configFileName
     */
    public function __construct(string $appPath, string $configFileName = Config::DEFAULT_CONFIG_FILE)
    {
        $this->configFileName = $configFileName;
        parent::__construct($appPath);
    }
    
    /**
     * Handle console command and return a code
     * 
     * @return int
     */
    public function handle()
    {
        $kernel = new Kernel($this);
        
        return $kernel->handle();
    }

    /**
     * {@inheritDoc}
     */
    public function description(): string
    {
        return 'InSegment Queue Workers Framework';
    }

    /**
     * {@inheritDoc}
     */
    protected function getConfigInstance(): ModelInterface
    {
        $configFileName = implode(DIRECTORY_SEPARATOR, [$this->getAppPath(), $this->configFileName]);
        $config = new Config($this->getFileReader(), $configFileName);
        $config->load();
        return $config;
    }
    
}
