<?php

namespace InSegment\WorkersFramework\Drivers\RabbitMq;

use Shavshukov\EventAppBase\Interfaces\ModelInterface;
use Shavshukov\EventAppBase\Model\ArrayModel;
use React\Promise;
use Bunny\Channel;
use Bunny\Message;

use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;

class QueueMessage implements QueueMessageInterface
{
    /**
     * Channel which has received message and will be used for acknowledgement
     *
     * @var \Bunny\Channel
     */
    protected $channel;
    
    /**
     * Bunny message
     *
     * @var \Bunny\Message
     */
    protected $message;
    
    /**
     * Name of the queue this message came from
     *
     * @var string
     */
    protected $queueName;
    
    /**
     * Whether the message was acknowledged
     *
     * @var bool
     */
    protected $isAcknowleged = false;
    
    /**
     * Acknowledge promise
     *
     * @var \React\Promise\PromiseInterface|null
     */
    protected $ackPromise = null;
    
    /**
     * Constructor
     * 
     * @param \Bunny\Message $message
     * @param \Bunny\Channel $channel
     */
    public function __construct(Message $message, Channel $channel, string $queueName)
    {
        $this->message = $message;
        $this->channel = $channel;
        $this->queueName = $queueName;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getConsumerTag(): ?string
    {
        return $this->message->consumerTag;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getContents(): string
    {
        return $this->message->content;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getHeaders(): ModelInterface
    {
        return ArrayModel::from($this->message->headers);
    }
    
    /**
     * {@inheritDoc}
     */
    public function getQueueName(): string
    {
        return $this->queueName;
    }
    
    /**
     * {@inheritDoc}
     */
    public function acknowlege()
    {
        if ($this->isAcknowleged) {
            return Promise\resolve();
        }
        
        if ($this->ackPromise !== null) {
            return $this->ackPromise;
        }
        
        return $this->ackPromise = $this->channel->ack($this->message)->then(function ($result) {
            $this->isAcknowleged = true;
            $this->ackPromise = null;
            return $result;
        }, function ($e) {
            $this->ackPromise = null;
            return $e;
        });
    }
    
    /**
     * {@inheritDoc}
     */
    public function fail()
    {
        return $this->channel->reject($this->message, false);
    }
    
}
