<?php

namespace InSegment\WorkersFramework\Models;

use Shavshukov\EventAppBase\Model\ArrayModel;

use InSegment\WorkersFramework\Interfaces\JobConfigInterface;
use InSegment\WorkersFramework\Interfaces\JobInterface;

class JobConfig extends ArrayModel implements JobConfigInterface
{
    
    /**
     * Name of the job
     *
     * @var string
     */
    protected $name;
    
    /**
     * Constructor
     * 
     * @param string $name
     * @param array $config
     */
    public function __construct(string $name, array $config)
    {
        $this->name = $name;
        $this->data = $config;
    }
    
    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $mandatoryFields = ['class'];
    
    /**
     * {@inheritDoc}
     */
    public function getClass(): string
    {
        return $this->get('class');
    }

    /**
     * {@inheritDoc}
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * {@inheritDoc}
     */
    public function getWeight(): int
    {
        return $this->get('weight', 1);
    }

    /**
     * {@inheritDoc}
     */
    public function getWorkers(): ?array
    {
        return $this->get('workers');
    }
    
    /**
     * {@inheritDoc}
     */
    public function createInstance(): JobInterface
    {
        $jobClass = $this->getClass();
        try {
            $jobInstance = new $jobClass();
            if (!$jobInstance instanceof JobInterface) {
                throw new \Exception("Cannot run job: {$jobClass} is not an implementor of JobInterface!");
            }
            
            return $jobInstance;
        } catch (\Throwable $e) {
            throw new \Exception("Cannot create an instance of {$jobClass}!", 0, $e);
        }
    }

}
